# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist

from driverng.astm import AstmMessageV2
from driverng.astm import HeaderSegment
from driverng.astm import Segment
from driverng.astm import HprimDate
from driverng.astm import HprimString
from driverng.astm import HprimForeign
from driverng.astm import HprimList
from driverng.astm import HprimMultiple


logger = logging.getLogger("driverng.drivers.sentifit270")


class Sentifit270SegmentH(HeaderSegment):
    analyzer = HprimString(5, default="Host")
    date = HprimDate(14, fmt_date="%Y%m%d%H%M%S")

    patients = HprimForeign("P", "patients")
    queries = HprimForeign("Q", "queries")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Sentifit270SegmentP(Segment):
    """The Patients segment
    """

    # "P"
    patient_id = HprimString(3)

    # can be empty
    patient_name = HprimString(6)

    # can be empty
    # for normal range, une "G2"
    birth_date = HprimDate(8, fmt_date="%Y%m%d")

    # can be empty
    # for normal range use:
    #   - "Unknown" (U)
    #   - "Male" (M) if "except F: Female" is selected
    sex = HprimString(9)

    # can be empty
    physician_name = HprimString(14)

    # can be empty
    spe1 = HprimString(15)

    # can be empty
    # size is 0, but stiil used for blood type:
    """Blood type correspondance
    +------+------------+
    | code | Blood Type |
    +------+------------+
    |  1   |   A Rh+    |
    +------+------------+
    |  2   |   A Rh-    |
    +------+------------+
    |  3   |   B Rh+    |
    +------+------------+
    |  4   |   B Rh-    |
    +------+------------+
    |  5   |   O Rh+    |
    +------+------------+
    |  6   |   O Rh-    |
    +------+------------+
    |  7   |   AB Rh+   |
    +------+------------+
    |  8   |   AB Rh-   |
    +------+------------+
    """
    spe2 = HprimString(16)

    # can be empty
    # service
    location = HprimString(26)

    orders = HprimForeign("O", "orders")


class Sentifit270SegmentO(Segment):
    """The orders segment
    """

    sample_id = HprimString(3)

    # 00        No order
    # 01-59     Clinical chemistry
    # 81        Serum information
    # universal_test_id
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))
    extraction_datetime = HprimDate(8, fmt_date="%Y%m%d%H%M%S")

    #  0 (and others): normal
    #  1: hight
    #  2. low
    extracted_volume = HprimString(10)

    # can be empty
    # default: 02
    specimen16 = HprimString(16, default="02")

    # can be empty
    # default: 02
    specimen17 = HprimString(17, default="02")

    results = HprimForeign("R", "results")


class Sentifit270SegmentR(Segment):
    """The result segment
    """

    """universal test ID ranges
     +----------+-------------------------------------------------------------+
     |00        |   No order                                                  |
     +----------+-------------------------------------------------------------+
     |01-59     |   Analyzer transmits Universal Test ID with zero suppression|
     +----------+-------------------------------------------------------------+
     |81        |   Serum information (H )                                    |
     +----------+-------------------------------------------------------------+
     |82        |   Serum information (L)                                     |
     +----------+-------------------------------------------------------------+
     |83        |   Serum information (I)                                     |
     +----------+-------------------------------------------------------------+
     |71-80     |   Calculated test                                           |
     +----------+-------------------------------------------------------------+
     |3011-3040 |   Calculated test                                           |
     +----------+-------------------------------------------------------------+
    """
    test = HprimString((3, 1))
    # universal_test_id
    code = HprimString((3, 4))

    # Résultat quantitatif
    value = HprimString((4, 1))
    # Résultat qualitif (Interprétation)
    value_quality = HprimString((4, 2))
    unit = HprimString(5)

    flag = HprimList(7)
    test_completion_datetime = HprimDate(13, fmt_date="%Y%m%d%H%M%S")


class Sentifit270SegmentC(Segment):
    """The comments segment
    """

    # input: 1 ; output: L
    source = HprimString(3)

    comment = HprimString(4)

    """What to do with comments
    +---------------+-----------------------+
    |Preceding msg  |  Type of comment      |
    +---------------+-----------------------+
    |    patient    |   G                   |
    +---------------+-----------------------+
    |    order      |   G                   |
    |               |  Reflect sample info  |
    +---------------+-----------------------+
    |    result     |   -                   |
    +---------------+-----------------------+
    |    scientific |   G                   |
    +---------------+-----------------------+
    |    manufacture|   G                   |
    +---------------+-----------------------+
    """
    type = HprimString(5)


class Sentifit270SegmentQ(Segment):

    starting_range = HprimString(3)

    # fixed to N
    req_info_status_code = HprimString(13)


class Sentifit270SegmentL(Segment):
    pass


class Sentifit270SegmentS(Segment):
    pass


class Sentifit270SegmentM(Segment):
    pass


class Driver(AstmMessageV2):

    SPEC = {
        "code": "sentifit270",
        "label": "Sentifit 270 SYSMEX",
    }

    SEGMENT_CLASS_MAP = {
        "H": Sentifit270SegmentH,
        "P": Sentifit270SegmentP,
        "O": Sentifit270SegmentO,
        "R": Sentifit270SegmentR,
        "C": Sentifit270SegmentC,
        "Q": Sentifit270SegmentQ,
        "L": Sentifit270SegmentL,

        #  not used
        "S": Sentifit270SegmentS,
        "M": Sentifit270SegmentM,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = Sentifit270SegmentH(type_segment="H", **kwargs)
        message.end = Sentifit270SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            physician_name = " ".join(([
                (physician.last_name, physician.first_name)
                for physician in patient.physicians
                if physician.type.lower() == 'attending'
            ] or [()])[0])
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                patient_name=" ".join((patient.last_name, patient.first_name)),
                birth_date=patient.birthdate,
                sex=patient.sex,
                physician_name=physician_name[:32],
                location=patient.service,
            )

            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    specimen16=sample.specimen,
                    specimen17=sample.specimen,
                    extraction_datetime=sample.collection_date,
                )
                for order in sample.orders:
                    test = seg_o.tests.add()
                    test.code = order.code
            for comment in patient.comments:
                seg_o.commentaires.add(
                    type="G",
                    comment=comment.text,
                    source="L",
                )

        return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    sample_id=seg_q.starting_range.strip(),
                    type="patient"
                )
            return worklist
        worklist = Worklist(context="result")
        for patient in message.patients:
            comments = []
            for order in patient.orders:
                # QC
                if patient.patient_id.startswith("970000"):
                    sample = worklist.get_or_create(
                        sample_id=order.sample_id,
                        control_lot=order.sample_id,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=order.sample_id,
                        type="patient"
                    )
                sample.specimen = (
                    order.specimen16 or
                    order.specimen17
                )
                comments.extend([
                    comment.comment
                    for comment in order.commentaires
                    if comment.type.lower() == "g" and comment.comment
                ])
                for result in order.results:
                    if result.value:
                        sample.results.add(
                            unit=result.unit,
                            flags=result.flag,
                            value=result.value,
                            code=result.test or result.code,
                            comments=comments,
                            date=result.test_completion_datetime
                        )
                    if result.value_quality:
                        sample.results.add(
                            unit=result.unit,
                            flags=result.flag,
                            value=result.value_quality,
                            code="{}QUAL".format(result.test or result.code),
                            comments=comments,
                            date=result.test_completion_datetime
                        )
        return worklist
