# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist

from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class StagoSegmentH(HeaderSegment):

    # TODO: automate->ident_serv_auto
    emetteur = HprimMultiple(5,
                             no_station=HprimString(1, default="2"),
                             version_protocole=HprimString(
                                 2, default="2.00"),
                             )

    # P = Production
    # Q = Controle Qualité
    # T = Formation
    # D = debug
    id_traitement = HprimString(12, default="P")

    no_version = HprimString(13, default="1.00")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True)


class StagoSegmentP(Segment):

    info = HprimMultiple(5, max_length=38,
                         info1=HprimString(1, max_length=16),
                         info2=HprimString(2, max_length=12),
                         info3=HprimString(3, max_length=6),
                         info4=HprimString(4, max_length=4),
                         )

    # Seulement pris en compte par le protocole étendu
    date_naissance = HprimDate(8, fmt_date="%Y%m%d")

    orders = HprimForeign("O", "patient")


class StagoSegmentO(Segment):

    # rack et N° lot Seulement en protocole étendu
    tube = HprimString(
        (3, 1), obligatoire=True)  # identification de l'échantillon
    rack = HprimString((3, 2))
    pos_rack = HprimString((3, 3))
    no_lot = HprimString((3, 4))

    tests = HprimList(
        5,
        basefield=HprimMultiple(
            0,
            identification=HprimString(1),
            nom_identification=HprimString(2),
            type_identification=HprimString(3),
            code=HprimString(4),
        ),
    )

    urgence = HprimString(6, max_length=1)

    results = HprimForeign("R", "order")


class StagoSegmentQ(Segment):

    tube = HprimString((3, 2), max_length=16)


class StagoSegmentR(Segment):

    test = HprimMultiple(3,
                         identification=HprimString(1),
                         nom_identification=HprimString(2),
                         type_identification=HprimString(3),
                         code=HprimString(4),
                         )
    valeur = HprimString(4)
    unite = HprimString(5)

    # F = Résultat finaux
    # C = Correction des résultats transmis précédemment
    # P = Résultat préliminaires
    # X = Ne peuvent être obtenus
    # I = En attente dans l'analyseur
    # S = Résultats partiels
    # M = En limite de sensibilité
    # R = Transmis précedemment
    # N = Ce résultat contient les informations pour lancer
    # une nouvelle demande
    # Q = Réponse à une demande non safisfaite
    # V = Vérifié / Approuvé par l'opérateur
    status = HprimString(9)

    date_fin_realisation = HprimDate(13)

    infos = HprimForeign("M", "result")


class StagoSegmentM(Segment):

    # 1 = À valider
    # 2 = Erreur technique
    # 3 = > M max
    # 4 = < M min
    # 5 = Ecart > tolérance
    # 6 = "Q IP" (sans plasma)
    # 8 = linéarité
    # A = validé
    # Code qui concerne le resultat commenté
    code_erreur = HprimString(3, max_length=1)

    # @ = Pas de code d'alarme
    # A = Résultat validé avec T > Tmax
    # B = Non utilisé
    # C = CQ hors normes ou non fait
    # D = CQ validation forcée
    # E = Aiguille n°3 détection de niveau non gérée
    # F = Aiguille n°2 détection de niveau non gérée
    # G = Aiguille n°1 détection de niveau non gérée
    # H = Résultat valeur en unité principale écrêtée
    # I = Résultat changement de dilution
    # J = Résultat test relancé
    # K = Tiroir réactifs: température hors limites
    # L = Seringue bras 3: date d'entretien dépassé
    # M = Seringue bras 2: Date d'entretien dépassé
    # N = Seringue bras 1: Date d'entretien dépassé
    # O = Ecart > tolérance sur mesure double
    # Code qui concerne l'état de l'automate
    # sans rapport direct avec le resultat commenté
    code_alarme = HprimString(4, max_length=1)


class StagoSegmentL(Segment):

    # N = Terminaison normale
    # T = Interrompue par l'émetteur
    # R = Interruption demandée par le récepteur
    # E = Erreur système inconnue
    # Q = Erreur dans la dernière demande d'informations
    # I = Pas d'informations disponibles à partir de la dernière demande
    # F = Dernière demande d'information traitée
    code = HprimString(3, max_length=1)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "stago",
        "label": "Stago",
    }
    SEGMENT_CLASS_MAP = {
        "H": StagoSegmentH,
        "P": StagoSegmentP,
        "O": StagoSegmentO,
        "Q": StagoSegmentQ,
        "R": StagoSegmentR,
        "M": StagoSegmentM,
        "L": StagoSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option("qc_prefix", "Préfixe QC"),
    ]

    def __init__(self, qc_prefix=None, **kwargs):
        self.qc_prefix = qc_prefix
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = StagoSegmentH(type_segment="H", **kwargs)
        message.end = StagoSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        msg.end.code = "N"
        for patient in worklist.get_astm_worklist(
                unique_together=["action", "priority"],
                action={"del": worklist.IGNORE, worklist.ELSE: None},
                priority={"A": "S", "S": "S", "R": "R", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add()
            seg_p.info.info1 = patient.last_name
            seg_p.info.info2 = patient.first_name
            if patient.birthdate and context["now"]:
                age = context["now"] - patient.birthdate
                seg_p.info.info3 = "{} ans".format(age.days // 365,)
            seg_p.info.info4 = patient.sex
            seg_p.date_naissance = patient.birthdate
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    tube=sample.sample_id,
                    urgence=priority,
                )
                for order in sample.orders:
                    test = seg_o.tests.add()
                    test.code = order.code

        if not len(msg.patients):
            return None
        else:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(sample_id=seg_q.tube, type="patient")
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if seg_o.rack and seg_o.pos_rack:
                    rack = "{}-{}".format(seg_o.rack, seg_o.pos_rack)
                else:
                    rack = None

                if (
                        message.id_traitement == "Q" or
                        self.qc_prefix and seg_o.tube.startswith(self.qc_prefix)
                ):
                    tube = "{}{}".format(seg_o.tube, seg_o.no_lot)
                    sample = worklist.get_or_create(
                        type="control",
                        sample_id=tube,
                        control_lot=tube,
                        control_name=seg_o.no_lot,
                        control_level=seg_o.tube,
                    )
                else:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.tube,
                        rack=rack,
                    )

                for seg_r in seg_o.results:

                    # Le segment M est REQUIS
                    alarmes = [seg_r.infos[0].code_alarme]

                    if seg_r.infos[0].code_erreur != "A":
                        # Il peut y avoir un code_alarme A
                        # (validé avec T > Tmax) qui fait
                        # doublon avec le code_erreur A (validé).
                        # Donc on ignore le code_erreur A
                        alarmes.append(seg_r.infos[0].code_erreur)

                    sample.results.add(
                        code=seg_r.test.code,
                        value=seg_r.valeur,
                        flags=alarmes,
                        date=seg_r.date_fin_realisation,
                    )
        return worklist
